# Generated by Django 2.1.14 on 2020-03-27 10:45

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0055_auto_20200325_1335'),
        ('djautotask', '0055_auto_20200325_0928'),
    ]

    operations = [
    ]
