# Generated by Django 4.0.7 on 2022-10-20 17:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0109_billingcode_billing_code_type'),
    ]

    operations = [
        migrations.AddField(
            model_name='account',
            name='phone',
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
    ]
