# Generated by Django 2.1.14 on 2020-08-07 15:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0070_auto_20200714_0807'),
    ]

    operations = [
        migrations.AddField(
            model_name='project',
            name='status_detail',
            field=models.CharField(blank=True, max_length=2000, null=True),
        ),
    ]
