# Generated by Django 2.1.11 on 2019-08-29 10:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='ticket',
            name='ticket_number',
            field=models.CharField(blank=True, max_length=50, null=True),
        ),
    ]
