# Generated by Django 4.0.7 on 2022-08-17 20:48

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0105_tasknote_created_by_contact'),
    ]

    operations = [
        migrations.AddField(
            model_name='ticketnote',
            name='created_by_contact',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='djautotask.contact'),
        ),
    ]
