# Generated by Django 3.1.1 on 2020-10-01 16:57

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0077_auto_20200924_1512'),
        ('djautotask', '0076_auto_20200925_1326'),
    ]

    operations = [
    ]
