# Generated by Django 2.1.14 on 2020-05-05 16:00

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0061_auto_20200416_1241'),
    ]

    operations = [
        migrations.AddField(
            model_name='task',
            name='department',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='djautotask.Department'),
        ),
    ]
