# Generated by Django 3.1.7 on 2021-07-27 09:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0093_auto_20210629_1204'),
    ]

    operations = [
        migrations.AddField(
            model_name='ticket',
            name='checklist_completed',
            field=models.PositiveSmallIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='ticket',
            name='checklist_total',
            field=models.PositiveSmallIntegerField(blank=True, null=True),
        ),
    ]
